#include "em3330.h"
#include "sfr_r835c.h"
#include "printser.h"

volatile unsigned long gulEMd0;
volatile unsigned char gucEMd1;
volatile unsigned char gfTelRdy = 0;

struct EMdata stSens[SENS_ANZ];
/**************************************
** init_struct
**
** clear ID and TimeOut value
** 22.09.2011
**************************************/
void em_initStruct(void)
{
	unsigned char ucCnt;

	for(ucCnt = 0; ucCnt < SENS_ANZ; ucCnt++)
	{
		stSens[ucCnt].uiID = 0;
		stSens[ucCnt].uiTimeOut = 0;
	}
}
/**************************************
** print all sensor data
**
** 22.09.2011
**************************************/
unsigned char em_printIndex(unsigned char ucIndex)
{
	unsigned char ucRet;

	if(stSens[ucIndex].uiID)
	{
		printser("ID: ");
		printser_4hex(stSens[ucIndex].uiID >> 2);

		printser("\tCH: ");
		printser_2hex(stSens[ucIndex].uiID & 3);

		printser("\tTemp: ");
		printser_dec(stSens[ucIndex].siTemp);

		printser("C\tHum: ");
		printser_dec(stSens[ucIndex].ucHum);

		printser("%\tTO: ");
		printser_dec(stSens[ucIndex].uiTimeOut);
		printser("\t(");
		printser_dec(stSens[ucIndex].uiDelta);
		printser(")\r\n");
		
		ucRet = 1;
	}
	else ucRet = 0;
	
	return ucRet;
}
/**************************************
** print all sensor data
**
** print data of one sensor in CSV style
** $[ID],[CH],[TEMP * 10],[HUM],[TIMEOUT],[DELTA];
** 22.09.2011
**************************************/
unsigned char em_printIndexCSV(unsigned char ucIndex)
{
	unsigned char ucRet;

	if(stSens[ucIndex].uiID)
	{
		printser_c('$');
		printser_dec(stSens[ucIndex].uiID >> 2);

		printser_c(',');
		printser_dec(stSens[ucIndex].uiID & 3);

		printser_c(',');
		printser_dec(stSens[ucIndex].siTemp);

		printser_c(',');
		printser_dec(stSens[ucIndex].ucHum);

		printser_c(',');
		printser_dec(stSens[ucIndex].uiTimeOut);
		printser_c(',');
		printser_dec(stSens[ucIndex].uiDelta);
		printser(";\r\n");
		
		ucRet = 1;
	}
	else ucRet = 0;
	
	return ucRet;
}
/**************************************
** get sensor data
**
** 22.09.2011
**************************************/
unsigned int em_getData(signed int *psiTemp, unsigned char *pucHum)
{
	unsigned int uiID;
	
	if(gfTelRdy)
	{
		*pucHum = gulEMd0 & 0xff;
		gulEMd0 >>= 8;

		*psiTemp = gulEMd0 & 0x0fff;
		if(*psiTemp & 0x0800)
			*psiTemp |= 0xf000;
		gulEMd0 >>= 12;


		uiID = gucEMd1 & 0x0f;
		uiID <<= 12;
		uiID |= gulEMd0 & 0xfff;

		gfTelRdy = 0;
	}
	else
	{
		uiID = 0;
	}
	return uiID;
}
/**************************************
** find index
**
** find ID index in structure or next empty if not in list
** return 0xff if structure is full
**
** 20.09.2011
**************************************/
unsigned char em_findIndex(unsigned int uiID)
{
	unsigned char ucCnt;
	
	ucCnt = 0;
	while( (ucCnt < SENS_ANZ) && (uiID != stSens[ucCnt].uiID) && (stSens[ucCnt].uiID != 0) )
	{
		ucCnt++;
	}
	if(ucCnt == SENS_ANZ)
		ucCnt = 0xff;			// structure full

	return ucCnt;
}
/**************************************
** remove ID
**
** remove ID from list by coping last index
** to the given index
**
** 20.09.2011
**************************************/
void em_removeIndex(unsigned char ucIndex1)
{
	unsigned char ucIndex2;
	
	ucIndex2 = em_findIndex(0xffff);
	if(ucIndex2 == 0xff)
		ucIndex2 = SENS_ANZ;

	if(ucIndex2 != ucIndex1)
	{
		ucIndex2--;
		stSens[ucIndex1].uiID = stSens[ucIndex2].uiID;
		stSens[ucIndex1].siTemp = stSens[ucIndex2].siTemp;
		stSens[ucIndex1].ucHum = stSens[ucIndex2].ucHum;
		stSens[ucIndex1].uiTimeOut = stSens[ucIndex2].uiTimeOut;
		stSens[ucIndex1].uiDelta = stSens[ucIndex2].uiDelta;
	}
	stSens[ucIndex2].uiID = 0;
	stSens[ucIndex2].uiDelta = 0;
}
/**************************************
** set Data
**
** find ID or next free place in list and store data
** return 0xff if list is full, else return index
**
** 20.09.2011
**************************************/
unsigned char em_setData(unsigned int uiID, signed int siTemp, unsigned char ucHum)
{
	unsigned char ucIndex;

	ucIndex = em_findIndex(uiID);
	if(ucIndex != 0xff)
	{
		stSens[ucIndex].uiID = uiID;
		stSens[ucIndex].siTemp = siTemp;
		stSens[ucIndex].ucHum = ucHum;
		stSens[ucIndex].uiDelta = SENS_TO - stSens[ucIndex].uiTimeOut;
		stSens[ucIndex].uiTimeOut = SENS_TO;
	}
	
	return ucIndex;
}
/**************************************
** EM decrement time out counter
** and remove sensor from list
** in case of time out
** 20.09.2011
**************************************/
void em_decTO(void)
{
	unsigned ucCnt = 0;

	while(stSens[ucCnt].uiID)
	{
		if(stSens[ucCnt].uiTimeOut)
			stSens[ucCnt].uiTimeOut--;
		else
			em_removeIndex(ucCnt);
		ucCnt++;
	}
}
/**************************************
** EM get Timeout of indexed Sensor
**
** 20.09.2011
**************************************/
unsigned int em_getTO(unsigned char ucIndex)
{
	unsigned int uiTO;
	
	if(ucIndex != 0xff)
	{
		uiTO = stSens[ucIndex].uiTimeOut;
	}
	else
		uiTO = 0;
	
	return uiTO;
}
/**************************************
** ISR INT0
**
** called on raising / falling edge of RF data
** 20.09.2011
**************************************/
#pragma interrupt/B ISR_int0(vect=29)
void ISR_int0(void)
{
	static unsigned long ulEMd0, ulEMd0_old;
	static unsigned char ucEMd1, ucEMd1_old;

	static unsigned char ucBitCnt = 40;
	unsigned char ucTRAsik;

	ucTRAsik = 0xff - tra;
	tra = 0xff;
	if(!tundf_tracr)
	{
#ifdef RX_INVERTED
		if(pol_int0ic)										// if falling edge (end of high pulse)
#else
		if(!pol_int0ic)
#endif
		{
// low (end of high)
			if( (ucTRAsik > RFTMIN_H) && (ucTRAsik < RFTMAX_H) )	// check if high pulse was in a valid range
			{
#ifdef RX_INVERTED
				pol_int0ic = 0;							// set polarity for rising edge (end of low phase)
#else
				pol_int0ic = 1;
#endif
			}
			else
			{
				ucBitCnt = 0;							// if high pulse was not valid clear bitcount (restart reception)
			}
		} // if(low)
		else
		{

/* high (end of low phase):
** - check if low phase was in a valid range
** - store bit in ucEMd1:ulEMd0 (8+32 bit)
** - if all 36 bit received copy data and set a signal
*/

			if( (ucTRAsik > RFTMIN_L) && (ucTRAsik < RFTMAX_L) && (ucBitCnt <36) ) // check low phase time
			{
				ucEMd1 <<= 1;								// rotate RX data
				if(ulEMd0 & 0x80000000)
					ucEMd1 |= 1;
				ulEMd0 <<= 1;
				if(ucTRAsik > RFTDIS)						// Bit value depends on length of low phase
					ulEMd0 |= 1;

				if(ucBitCnt == 35)							// if all done
				{
					ucEMd1 &= 0x0f;
					if((ulEMd0 == ulEMd0_old) && (ucEMd1 == ucEMd1_old)) // if equal with last reception
					{
						if(!gfTelRdy)						// store only if data are free
						{
							gulEMd0 = ulEMd0;
							gucEMd1 = ucEMd1;
							gfTelRdy = 1;					// set signal
						}
					}
					ulEMd0_old = ulEMd0;					// copy new data to compare data
					ucEMd1_old = ucEMd1;
				}
				ucBitCnt++;
			}
			else											// low phase was out of range
			{
				ucBitCnt = 0;								// -> clear bit count
			}

#ifdef RX_INVERTED
			pol_int0ic = 0;									// change polarity for falling edge
#else
			pol_int0ic = 1;
#endif
		} // if(high)
	}// TUNDF
	else							// time out:
	{
		tundf_tracr = 0;
		ucBitCnt = 0;				// clear bitcount
	}
}
